@@ -30,6 +30,7 @@
 #include "caf/fwd.hpp"
 #include "caf/inspector_access_base.hpp"
 #include "caf/inspector_access_type.hpp"
+#include "caf/intrusive_cow_ptr.hpp"
 #include "caf/intrusive_ptr.hpp"
 #include "caf/optional.hpp"
 #include "caf/sec.hpp"
@@ -57,26 +58,6 @@ constexpr auto always_true = always_true_t{};
 template <class>
 constexpr bool assertion_failed_v = false;
 
-// TODO: remove with CAF 0.19
-template <class T, class Inspector, class Obj>
-class has_static_apply {
-private:
-  template <class U>
-  static auto sfinae(Inspector* f, Obj* x) -> decltype(U::apply(*f, *x));
-
-  template <class U>
-  static auto sfinae(...) -> std::false_type;
-
-  using sfinae_type = decltype(sfinae<T>(nullptr, nullptr));
-
-public:
-  static constexpr bool value
-    = !std::is_same<sfinae_type, std::false_type>::value;
-};
-
-template <class T, class Inspector, class Obj>
-constexpr bool has_static_apply_v = has_static_apply<T, Inspector, Obj>::value;
-
 // -- loading ------------------------------------------------------------------
 
 // Converts a setter that returns void, error or bool to a sync function object
@@ -105,17 +86,8 @@ auto bind_setter(Inspector& f, Set& set, ValueType& tmp) {
   }
 }
 
-// TODO: remove with CAF 0.19
 template <class Inspector, class T>
-[[deprecated("please provide apply instead of apply_object/apply_value")]] //
-std::enable_if_t<!has_static_apply_v<inspector_access<T>, Inspector, T>, bool>
-load(Inspector& f, T& x, inspector_access_type::specialization) {
-  return inspector_access<T>::apply_value(f, x);
-}
-
-template <class Inspector, class T>
-std::enable_if_t<has_static_apply_v<inspector_access<T>, Inspector, T>, bool>
-load(Inspector& f, T& x, inspector_access_type::specialization) {
+bool load(Inspector& f, T& x, inspector_access_type::specialization) {
   return inspector_access<T>::apply(f, x);
 }
 
@@ -206,17 +178,8 @@ bool load_field(Inspector& f, string_view field_name, T& x, IsValid& is_valid,
 
 // -- saving -------------------------------------------------------------------
 
-// TODO: remove with CAF 0.19
 template <class Inspector, class T>
-[[deprecated("please provide apply instead of apply_object/apply_value")]] //
-std::enable_if_t<!has_static_apply_v<inspector_access<T>, Inspector, T>, bool>
-save(Inspector& f, T& x, inspector_access_type::specialization) {
-  return inspector_access<T>::apply_value(f, x);
-}
-
-template <class Inspector, class T>
-std::enable_if_t<has_static_apply_v<inspector_access<T>, Inspector, T>, bool>
-save(Inspector& f, T& x, inspector_access_type::specialization) {
+bool save(Inspector& f, T& x, inspector_access_type::specialization) {
   return inspector_access<T>::apply(f, x);
 }
 
@@ -329,11 +292,23 @@ struct inspector_access;
 
 // -- inspection support for optional values -----------------------------------
 
+struct optional_inspector_traits_base {
+  template <class T>
+  static auto& deref_load(T& x) {
+    return *x;
+  }
+
+  template <class T>
+  static auto& deref_save(T& x) {
+    return *x;
+  }
+};
+
 template <class T>
 struct optional_inspector_traits;
 
 template <class T>
-struct optional_inspector_traits<optional<T>> {
+struct optional_inspector_traits<optional<T>> : optional_inspector_traits_base {
   using container_type = optional<T>;
 
   using value_type = T;
@@ -345,7 +320,8 @@ struct optional_inspector_traits<optional<T>> {
 };
 
 template <class T>
-struct optional_inspector_traits<intrusive_ptr<T>> {
+struct optional_inspector_traits<intrusive_ptr<T>>
+  : optional_inspector_traits_base {
   using container_type = intrusive_ptr<T>;
 
   using value_type = T;
@@ -357,7 +333,28 @@ struct optional_inspector_traits<intrusive_ptr<T>> {
 };
 
 template <class T>
-struct optional_inspector_traits<std::unique_ptr<T>> {
+struct optional_inspector_traits<intrusive_cow_ptr<T>> {
+  using container_type = intrusive_cow_ptr<T>;
+
+  using value_type = T;
+
+  template <class... Ts>
+  static void emplace(container_type& container, Ts&&... xs) {
+    container.reset(new T(std::forward<Ts>(xs)...));
+  }
+
+  static value_type& deref_load(container_type& x) {
+    return x.unshared();
+  }
+
+  static const value_type& deref_save(container_type& x) {
+    return *x;
+  }
+};
+
+template <class T>
+struct optional_inspector_traits<std::unique_ptr<T>>
+  : optional_inspector_traits_base {
   using container_type = std::unique_ptr<T>;
 
   using value_type = T;
@@ -369,7 +366,8 @@ struct optional_inspector_traits<std::unique_ptr<T>> {
 };
 
 template <class T>
-struct optional_inspector_traits<std::shared_ptr<T>> {
+struct optional_inspector_traits<std::shared_ptr<T>>
+  : optional_inspector_traits_base {
   using container_type = std::shared_ptr<T>;
 
   using value_type = T;
@@ -394,23 +392,11 @@ struct optional_inspector_access {
     return f.object(x).fields(f.field("value", x));
   }
 
-  template <class Inspector>
-  [[deprecated("use apply instead")]] static bool
-  apply_object(Inspector& f, container_type& x) {
-    return apply(f, x);
-  }
-
-  template <class Inspector>
-  [[deprecated("use apply instead")]] static bool
-  apply_value(Inspector& f, container_type& x) {
-    return apply(f, x);
-  }
-
   template <class Inspector>
   static bool
   save_field(Inspector& f, string_view field_name, container_type& x) {
     auto is_present = [&x] { return static_cast<bool>(x); };
-    auto get = [&x]() -> decltype(auto) { return *x; };
+    auto get = [&x]() -> decltype(auto) { return traits::deref_save(x); };
     return detail::save_field(f, field_name, is_present, get);
   }
 
@@ -426,7 +412,8 @@ struct optional_inspector_access {
                          SyncValue& sync_value) {
     traits::emplace(x);
     auto reset = [&x] { x.reset(); };
-    return detail::load_field(f, field_name, *x, is_valid, sync_value, reset);
+    return detail::load_field(f, field_name, traits::deref_load(x), is_valid,
+                              sync_value, reset);
   }
 
   template <class Inspector, class IsValid, class SyncValue, class SetFallback>
@@ -434,8 +421,8 @@ struct optional_inspector_access {
                          container_type& x, IsValid& is_valid,
                          SyncValue& sync_value, SetFallback& set_fallback) {
     traits::emplace(x);
-    return detail::load_field(f, field_name, *x, is_valid, sync_value,
-                              set_fallback);
+    return detail::load_field(f, field_name, traits::deref_load(x), is_valid,
+                              sync_value, set_fallback);
   }
 };
 
@@ -449,7 +436,8 @@ struct inspector_access<optional<T>> : optional_inspector_access<optional<T>> {
 #ifdef CAF_HAS_STD_OPTIONAL
 
 template <class T>
-struct optional_inspector_traits<std::optional<T>> {
+struct optional_inspector_traits<std::optional<T>>
+  : optional_inspector_traits_base {
   using container_type = std::optional<T>;
 
   using value_type = T;
@@ -478,8 +466,6 @@ struct inspector_access<std::unique_ptr<error::data>>
 
 // -- inspection support for std::byte -----------------------------------------
 
-#ifdef __cpp_lib_byte
-
 template <>
 struct inspector_access<std::byte> : inspector_access_base<std::byte> {
   template <class Inspector>
@@ -491,8 +477,6 @@ struct inspector_access<std::byte> : inspector_access_base<std::byte> {
   }
 };
 
-#endif
-
 // -- inspection support for variant<Ts...> ------------------------------------
 
 template <class T>
@@ -554,18 +538,6 @@ struct variant_inspector_access {
     return f.object(x).fields(f.field("value", x));
   }
 
-  template <class Inspector>
-  [[deprecated("use apply instead")]] static bool
-  apply_object(Inspector& f, T& x) {
-    return apply(f, x);
-  }
-
-  template <class Inspector>
-  [[deprecated("use apply instead")]] static bool
-  apply_value(Inspector& f, T& x) {
-    return apply(f, x);
-  }
-
   template <class Inspector>
   static bool save_field(Inspector& f, string_view field_name, value_type& x) {
     auto g = [&f](auto& y) { return detail::save(f, y); };
@@ -760,18 +732,6 @@ struct inspector_access<std::chrono::duration<Rep, Period>>
       return f.apply(get, set);
     }
   }
-
-  template <class Inspector>
-  [[deprecated("use apply instead")]] static bool
-  apply_object(Inspector& f, value_type& x) {
-    return apply(f, x);
-  }
-
-  template <class Inspector>
-  [[deprecated("use apply instead")]] static bool
-  apply_value(Inspector& f, value_type& x) {
-    return apply(f, x);
-  }
 };
 
 template <class Duration>
@@ -802,35 +762,6 @@ struct inspector_access<
       return f.apply(get, set);
     }
   }
-
-  template <class Inspector>
-  [[deprecated("use apply instead")]] static bool
-  apply_object(Inspector& f, value_type& x) {
-    return apply(f, x);
-  }
-
-  template <class Inspector>
-  [[deprecated("use apply instead")]] static bool
-  apply_value(Inspector& f, value_type& x) {
-    return apply(f, x);
-  }
-};
-
-// -- deprecated API -----------------------------------------------------------
-
-template <class T>
-struct default_inspector_access : inspector_access_base<T> {
-  template <class Inspector>
-  [[deprecated("call f.apply(x) instead")]] static bool
-  apply_object(Inspector& f, T& x) {
-    return f.apply(x);
-  }
-
-  template <class Inspector>
-  [[deprecated("call f.apply(x) instead")]] static bool
-  apply_value(Inspector& f, T& x) {
-    return f.apply(x);
-  }
 };
 
 } // namespace caf
