@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #pragma once
 
@@ -59,10 +59,10 @@ public:
     ~Exception() override;
 
     /** Returns the message associated with the exception. */
-    auto description() const { return _description; }
+    const auto& description() const { return _description; }
 
     /** Returns the location associated with the exception. */
-    auto location() const { return _location; }
+    const auto& location() const { return _location; }
 
     /**
      * Returns a stack backtrace captured at the time the exception was
@@ -196,6 +196,11 @@ HILTI_EXCEPTION(UnicodeError, RuntimeError)
  */
 HILTI_EXCEPTION(UnsetOptional, RuntimeError)
 
+/**
+ * Exception reflecting an access to an unset tuple element.
+ */
+HILTI_EXCEPTION(UnsetTupleElement, RuntimeError)
+
 /**
  * Exception triggered by member access to fields that don't hold the value.
  */
