@@ -1,8 +1,7 @@
-// Copyright (c) 2020-2023 by the Zeek Project. See LICENSE for details.
+// Copyright (c) 2020-now by the Zeek Project. See LICENSE for details.
 
 #pragma once
 
-#include <memory>
 #include <utility>
 
 #include <hilti/rt/types/port.h>
@@ -21,7 +20,7 @@ public:
 
     node::Properties properties() const final {
         auto p = node::Properties{{"value", to_string(_value)}};
-        return Ctor::properties() + p;
+        return Ctor::properties() + std::move(p);
     }
 
     static auto create(ASTContext* ctx, hilti::rt::Port v, const Meta& meta = {}) {
