@@ -16,3 +16,10 @@ node::Properties declaration::Function::properties() const {
 
     return Declaration::properties() + std::move(p);
 }
+
+ID declaration::Function::functionID(ASTContext* ctx) {
+    if ( auto* prototype = ctx->lookup(linkedPrototypeIndex()) )
+        return prototype->fullyQualifiedID();
+
+    return fullyQualifiedID();
+}
