@@ -41,8 +41,12 @@ struct Printer : visitor::PreOrder {
             return;
         }
 
-        if ( ftype.flavor() != type::function::Flavor::Standard )
-            _out << to_string(ftype.flavor()) << ' ';
+        _out << to_string(ftype.flavor()) << ' ';
+
+        if ( ftype.flavor() == type::function::Flavor::Function ) {
+            if ( ftype.callingConvention() != type::function::CallingConvention::Standard )
+                _out << to_string(ftype.callingConvention()) << ' ';
+        }
 
         _out << ftype.result() << ' ';
 
@@ -65,9 +69,9 @@ struct Printer : visitor::PreOrder {
         switch ( l ) {
             case declaration::Linkage::Init: return "init ";
             case declaration::Linkage::PreInit: return "preinit ";
-            case declaration::Linkage::Struct: return "method ";
-            case declaration::Linkage::Private: return ""; // That's the default.
             case declaration::Linkage::Public: return "public ";
+            case declaration::Linkage::Struct: [[fallthrough]];
+            case declaration::Linkage::Private: return ""; // That's the default.
             default: util::cannotBeReached();
         }
     }
@@ -116,9 +120,6 @@ struct Printer : visitor::PreOrder {
     }
 
     void operator()(Function* n) final {
-        if ( n->callingConvention() != function::CallingConvention::Standard )
-            _out << to_string(n->callingConvention()) << ' ';
-
         printFunctionType(*n->ftype(), n->id());
 
         if ( auto attrs = n->attributes()->attributes(); ! attrs.empty() )
@@ -211,7 +212,7 @@ struct Printer : visitor::PreOrder {
 
     void operator()(ctor::Error* n) final { _out << "error(\"" << n->value() << "\")"; }
 
-    void operator()(ctor::Exception* n) final { _out << "exception(" << n->value() << ")"; }
+    void operator()(ctor::Exception* n) final { _out << n->value(); }
 
     void operator()(ctor::Interval* n) final { _out << "interval_ns(" << n->value().nanoseconds() << ")"; }
 
@@ -311,7 +312,7 @@ struct Printer : visitor::PreOrder {
 
     void operator()(ctor::WeakReference* n) final { _out << "Null"; }
 
-    void operator()(ctor::ValueReference* n) final { _out << "value_ref(" << n->expression() << ')'; }
+    void operator()(ctor::ValueReference* n) final { _out << n->expression(); }
 
     ////// Declarations
 
@@ -329,11 +330,13 @@ struct Printer : visitor::PreOrder {
     void operator()(declaration::Field* n) final {
         _out << "    ";
 
-        if ( auto ft = n->type()->type()->tryAs<type::Function>() ) {
+        if ( auto* ft = n->type()->type()->tryAs<type::Function>() ) {
             _out << to_string(ft->flavor()) << " ";
 
-            if ( auto cc = n->callingConvention(); cc && *cc != function::CallingConvention::Standard )
-                _out << to_string(*cc) << ' ';
+            if ( ft->flavor() == type::function::Flavor::Function ) {
+                if ( auto cc = ft->callingConvention(); cc != type::function::CallingConvention::Standard )
+                    _out << to_string(cc) << ' ';
+            }
 
             _out << ft->result() << " " << n->id() << "(" << std::make_pair(ft->parameters(), ", ") << ")";
         }
@@ -341,10 +344,10 @@ struct Printer : visitor::PreOrder {
         else
             _out << n->type() << ' ' << n->id();
 
-        if ( auto attrs = n->attributes(); ! attrs->attributes().empty() )
+        if ( auto* attrs = n->attributes(); ! attrs->attributes().empty() )
             _out << ' ' << attrs;
 
-        if ( auto f = n->inlineFunction(); f && f->body() ) {
+        if ( auto* f = n->inlineFunction(); f && f->body() ) {
             const auto& block = f->body()->tryAs<statement::Block>();
             if ( block && block->statements().empty() ) {
                 _out << " {}";
@@ -370,7 +373,7 @@ struct Printer : visitor::PreOrder {
         if ( n->default_() )
             _out << " = " << n->default_();
 
-        if ( auto attrs = n->attributes(); ! attrs->attributes().empty() )
+        if ( auto* attrs = n->attributes(); ! attrs->attributes().empty() )
             _out << ' ' << attrs;
     }
 
@@ -389,10 +392,6 @@ struct Printer : visitor::PreOrder {
         }
 
         _out << linkage(n->linkage());
-
-        if ( n->linkage() != declaration::Linkage::Struct )
-            _out << "function ";
-
         _out << n->function();
     }
 
@@ -415,7 +414,7 @@ struct Printer : visitor::PreOrder {
         _out.setExpandSubsequentType(true);
         _out << n->type();
 
-        if ( auto attrs = n->attributes(); ! attrs->attributes().empty() )
+        if ( auto* attrs = n->attributes(); ! attrs->attributes().empty() )
             _out << ' ' << attrs;
 
         _out << ';';
@@ -434,8 +433,20 @@ struct Printer : visitor::PreOrder {
         if ( n->typeArguments().size() )
             _out << '(' << std::make_pair(n->typeArguments(), ", ") << ')';
 
-        if ( n->init() )
-            _out << " = " << n->init();
+        // We use void expressions as a hint for the initialization mechanism
+        // to use in C++ codegen. These expressions have no actual equivalent
+        // in the syntax.
+        //
+        // To still somehow capture them in HILTI output render them by
+        // declaring a variable with no constructor arguments; this is distinct
+        // from HILTI default initialization which uses assignment syntax. This
+        // is not 100% equivalent, but allows rendering valid code.
+        if ( auto* init = n->init() ) {
+            if ( init->isA<expression::Void>() )
+                _out << "()";
+            else
+                _out << " = " << init;
+        }
     }
 
     void operator()(declaration::GlobalVariable* n) final {
@@ -493,7 +504,7 @@ struct Printer : visitor::PreOrder {
         _out << ']';
     }
 
-    void operator()(expression::LogicalAnd* n) final { _out << n->op0() << " &* " << n->op1(); }
+    void operator()(expression::LogicalAnd* n) final { _out << n->op0() << " && " << n->op1(); }
 
     void operator()(expression::LogicalNot* n) final { _out << "! " << n->expression(); }
 
@@ -552,7 +563,7 @@ struct Printer : visitor::PreOrder {
     }
 
     void operator()(statement::Block* n) final {
-        if ( _out.indent() == 0 || n->statements().size() > 1 )
+        if ( _out.indent() == 0 || n->statements().size() != 1 )
             _out << "{";
 
         _out.endLine();
@@ -573,7 +584,7 @@ struct Printer : visitor::PreOrder {
 
         _out.decrementIndent();
 
-        if ( _out.indent() == 0 || n->statements().size() > 1 ) {
+        if ( _out.indent() == 0 || n->statements().size() != 1 ) {
             _out.beginLine();
             _out << "}";
             _out.endLine();
@@ -632,10 +643,10 @@ struct Printer : visitor::PreOrder {
         _out.beginLine();
         _out << "if ( ";
 
-        if ( auto e = n->init() )
+        if ( auto* e = n->init() )
             _out << e << "; ";
 
-        if ( auto e = n->condition() )
+        if ( auto* e = n->condition() )
             _out << e;
 
         _out << " ) " << n->true_();
@@ -658,7 +669,7 @@ struct Printer : visitor::PreOrder {
         _out.beginLine();
         _out << "return";
 
-        if ( auto e = n->expression() )
+        if ( auto* e = n->expression() )
             _out << ' ' << e;
 
         _out << ';';
@@ -701,7 +712,7 @@ struct Printer : visitor::PreOrder {
         _out.beginLine();
         _out << "throw";
 
-        if ( auto e = n->expression() )
+        if ( auto* e = n->expression() )
             _out << fmt(" %s", *e);
 
         _out << ";";
@@ -712,7 +723,7 @@ struct Printer : visitor::PreOrder {
         _out.beginLine();
         _out << "catch ";
 
-        if ( auto p = n->parameter() )
+        if ( auto* p = n->parameter() )
             _out << "( " << p << " ) ";
 
         _out << n->body();
@@ -733,10 +744,10 @@ struct Printer : visitor::PreOrder {
         _out.beginLine();
         _out << "while ( ";
 
-        if ( auto e = n->init() )
+        if ( auto* e = n->init() )
             _out << e << "; ";
 
-        if ( auto e = n->condition() )
+        if ( auto* e = n->condition() )
             _out << e;
 
         _out << " ) " << n->body();
@@ -773,7 +784,7 @@ struct Printer : visitor::PreOrder {
         else
             _out << fmt("%d..%d", n->lower(), n->upper());
 
-        if ( auto attrs = n->attributes(); ! attrs->attributes().empty() )
+        if ( auto* attrs = n->attributes(); ! attrs->attributes().empty() )
             _out << ' ' << attrs;
 
         _out << ";" << _out.newline();
@@ -833,7 +844,7 @@ struct Printer : visitor::PreOrder {
 
         _out.setExpandSubsequentType(false);
 
-        if ( auto t = n->baseType(); t && ! t->isA<type::Unknown>() ) {
+        if ( auto* t = n->baseType(); t && ! t->isA<type::Unknown>() ) {
             _out << "[exception :";
 
             if ( auto id = t->typeID() )
@@ -1121,6 +1132,7 @@ void printer::print(std::ostream& out, Node* root, bool compact, bool user_visib
         auto data = buffer.str();
         data = util::trim(data);
         data = util::replace(data, "\n", " ");
+        // NOLINTNEXTLINE(modernize-use-ranges)
         data.erase(std::unique(data.begin(), data.end(), [](char a, char b) { return a == ' ' && b == ' '; }),
                    data.end());
         out << data;
@@ -1134,11 +1146,11 @@ void printer::print(std::ostream& out, Node* root, bool compact, bool user_visib
 void printer::Stream::_print(Node* root) {
     util::timing::Collector _("hilti/printer");
 
-    for ( auto& p : plugin::registry().plugins() ) {
+    for ( const auto& p : plugin::registry().plugins() ) {
         if ( ! p.ast_print )
             continue;
 
-        auto* prev = std::exchange(detail::State::current->current_plugin, &p);
+        const auto* prev = std::exchange(detail::State::current->current_plugin, &p);
         auto _ = util::scope_exit([&]() { detail::State::current->current_plugin = prev; });
 
         if ( (*p.ast_print)(root, *this) )
