@@ -17,6 +17,13 @@ zeek.on('zeek_init', () => {
     zeek.print(`Caught it: ${error}`);
   }
 
+  try {
+
+    zeek.invoke('Test::test_add', ["a", "b"]);
+  } catch (error) {
+    zeek.print(`Caught it: ${error}`);
+  }
+
   try {
     zeek.invoke('Test::test_add', [{"a": "b"}, {"a": "b"}]);
   } catch (error) {
@@ -47,6 +54,18 @@ zeek.on('zeek_init', () => {
     zeek.print(`Caught it: ${error}`);
   }
 
+  try {
+    zeek.invoke('Test::test_add', 0);
+  } catch (error) {
+    zeek.print(`Caught it: ${error}`);
+  }
+
+  try {
+    zeek.invoke('Test::test_add', {});
+  } catch (error) {
+    zeek.print(`Caught it: ${error}`);
+  }
+
 });
 @TEST-END-FILE
 
