@@ -3,7 +3,7 @@
 #include "zeek/ScannedFile.h"
 
 #include <cerrno>
-#include <climits> // for PATH_MAX
+#include <filesystem>
 
 #include "zeek/DebugLogger.h"
 #include "zeek/Reporter.h"
@@ -23,7 +23,7 @@ ScannedFile::ScannedFile(int arg_include_level, std::string arg_name, bool arg_s
         canonical_path = canonical_stdin_path;
     else if ( ! arg_is_canonical ) {
         std::error_code ec;
-        auto canon = filesystem::canonical(name, ec);
+        auto canon = std::filesystem::canonical(name, ec);
         if ( ec )
             zeek::reporter->FatalError("failed to get canonical path of %s: %s", name.data(), ec.message().c_str());
 
@@ -50,6 +50,6 @@ bool ScannedFile::AlreadyScanned() const {
 SignatureFile::SignatureFile(std::string file) : file(std::move(file)) {}
 
 SignatureFile::SignatureFile(std::string file, std::string full_path, Location load_location)
-    : file(std::move(file)), full_path(std::move(full_path)), load_location(std::move(load_location)) {}
+    : file(std::move(file)), full_path(std::move(full_path)), load_location(load_location) {}
 
 } // namespace zeek::detail
