@@ -2,9 +2,8 @@
 
 #include "zeek/Type.h"
 
-#include "zeek/zeek-config.h"
-
-#include <list>
+#include <algorithm>
+#include <cinttypes>
 #include <map>
 #include <string>
 #include <unordered_set>
@@ -71,6 +70,7 @@ Type::Type(TypeTag t, bool arg_base_type)
       is_network_order(zeek::is_network_order(t)),
       base_type(arg_base_type) {}
 
+// NOLINTNEXTLINE(cppcoreguidelines-macro-usage)
 #define CHECK_TYPE_TAG(tag_type, func_name) CHECK_TAG(tag, tag_type, func_name, type_name)
 
 const TypeList* Type::AsTypeList() const {
@@ -424,8 +424,7 @@ static bool is_supported_index_type(const TypePtr& t, const char** tname, std::u
 
     switch ( tag ) {
         // Allow functions, since they can be compared for Func* pointer equality.
-        case TYPE_FUNC: return true;
-
+        case TYPE_FUNC:
         case TYPE_PATTERN: return true;
 
         case TYPE_RECORD: {
@@ -758,9 +757,7 @@ void FuncType::DescribeReST(ODesc* d, bool roles_only) const {
 void FuncType::AddPrototype(Prototype p) { prototypes.emplace_back(std::move(p)); }
 
 std::optional<FuncType::Prototype> FuncType::FindPrototype(const RecordType& args) const {
-    for ( auto i = 0u; i < prototypes.size(); ++i ) {
-        const auto& p = prototypes[i];
-
+    for ( const auto& p : prototypes ) {
         if ( args.NumFields() != p.args->NumFields() )
             continue;
 
@@ -906,7 +903,7 @@ public:
         auto v = init_expr->Eval(nullptr);
         if ( ! v ) {
             reporter->Error("failed &default in record creation");
-            return ZVal();
+            return {};
         }
 
         if ( coerce_type )
@@ -915,10 +912,39 @@ public:
         else if ( init_type->Tag() == TYPE_VECTOR )
             concretize_if_unspecified(cast_intrusive<VectorVal>(v), init_type->Yield());
 
-        return ZVal(v, init_type);
+        return {v, init_type};
     }
 
-    bool IsDeferrable() const override { return false; }
+    bool IsDeferrable() const override {
+        if ( init_expr->Tag() == EXPR_RECORD_CONSTRUCTOR ) {
+            // Special-case deferrable record construction.
+            auto rce = zeek::cast_intrusive<RecordConstructorExpr>(init_expr);
+            auto rt = rce->GetType<zeek::RecordType>();
+
+            // The empty constructor_list check here is a short-cut: If the
+            // constructor expression contained only further const expressions
+            // or only further deferrable record constructors, this could be
+            // more aggressively deferring initializations.
+            auto constructor_list = rce->Op();
+            return rt->IsDeferrable() && constructor_list->Exprs().empty();
+        }
+
+        // Allow deferring fairly common &default=vector() field initialization...
+        if ( init_expr->Tag() == EXPR_VECTOR_CONSTRUCTOR ) {
+            auto vce = zeek::cast_intrusive<VectorConstructorExpr>(init_expr);
+            return vce->GetType<zeek::VectorType>()->IsUnspecifiedVector();
+        }
+
+        // ...and also &default=table() and &default=set().
+        if ( init_expr->Tag() == EXPR_TABLE_COERCE || init_expr->Tag() == EXPR_TABLE_CONSTRUCTOR ||
+             init_expr->Tag() == EXPR_SET_CONSTRUCTOR ) {
+            auto une = zeek::cast_intrusive<UnaryExpr>(init_expr);
+            return une->Op()->GetType()->Tag() == TYPE_TABLE &&
+                   une->Op()->GetType<zeek::TableType>()->IsUnspecifiedTable();
+        }
+
+        return false;
+    }
 
     ExprPtr InitExpr() const override { return init_expr; }
 
@@ -934,7 +960,7 @@ class RecordFieldInit final : public FieldInit {
 public:
     RecordFieldInit(RecordTypePtr _init_type) : init_type(std::move(_init_type)) {}
 
-    ZVal Generate() const override { return ZVal(new RecordVal(init_type)); }
+    ZVal Generate() const override { return {new RecordVal(init_type)}; }
 
     bool IsDeferrable() const override {
         assert(! run_state::is_parsing);
@@ -952,7 +978,7 @@ public:
     TableFieldInit(TableTypePtr _init_type, detail::AttributesPtr _attrs)
         : init_type(std::move(_init_type)), attrs(std::move(_attrs)) {}
 
-    ZVal Generate() const override { return ZVal(new TableVal(init_type, attrs)); }
+    ZVal Generate() const override { return {new TableVal(init_type, attrs)}; }
 
 private:
     TableTypePtr init_type;
@@ -965,7 +991,7 @@ class VectorFieldInit final : public FieldInit {
 public:
     VectorFieldInit(VectorTypePtr _init_type) : init_type(std::move(_init_type)) {}
 
-    ZVal Generate() const override { return ZVal(new VectorVal(init_type)); }
+    ZVal Generate() const override { return {new VectorVal(init_type)}; }
 
 private:
     VectorTypePtr init_type;
@@ -995,10 +1021,29 @@ private:
     void OptimizeCreationInits(RecordType* rt) {
         int i = 0;
         for ( auto& ci : rt->creation_inits ) {
-            if ( ! ci.second->IsDeferrable() )
+            if ( ! ci.second->IsDeferrable() ) {
+                // A non-deferrable field with a &default attribute is expected to also exist in deferred_inits
+                // such that re-initialization after deletion of the field works.
+                if ( rt->FieldDecl(ci.first)->GetAttr(detail::ATTR_DEFAULT) != detail::Attr::nil ) {
+                    if ( ! rt->deferred_inits[ci.first] )
+                        zeek::reporter->InternalError("non-deferrable field %s$%s with &default not in deferred_inits",
+                                                      rt->GetName().c_str(), rt->FieldName(i));
+
+                    else if ( rt->deferred_inits[ci.first] != ci.second )
+                        zeek::reporter->InternalError("non-deferrable field %s$%s with &default has inconsistent inits",
+                                                      rt->GetName().c_str(), rt->FieldName(i));
+                }
+
                 rt->creation_inits[i++] = std::move(ci);
+            }
             else {
-                assert(! rt->deferred_inits[ci.first]);
+                // If deferred_inits already has a value, it should be the same as the one
+                // stored in creation_inits. This happens for deferrable record fields
+                // that have a &default attribute.
+                if ( rt->deferred_inits[ci.first] && rt->deferred_inits[ci.first] != ci.second )
+                    zeek::reporter->InternalError("deferrable field %s$%s has inconsistent inits",
+                                                  rt->GetName().c_str(), rt->FieldName(i));
+
                 rt->deferred_inits[ci.first].swap(ci.second);
             }
         }
@@ -1090,8 +1135,13 @@ void RecordType::AddField(unsigned int field, const TypeDecl* td) {
         }
 
         else {
-            auto efi = std::make_shared<detail::ExprFieldInit>(def_expr, type);
-            creation_inits.emplace_back(field, std::move(efi));
+            // Note that init is placed into creation_inits and deferred_inits
+            // such that accessing a record field after deleting it will run the
+            // &default expression to re-initialize it again.
+            //
+            // Also see RecordType::CreationInitsOptimizer.
+            init = std::make_shared<detail::ExprFieldInit>(def_expr, type);
+            creation_inits.emplace_back(field, init);
         }
     }
 
@@ -1216,7 +1266,12 @@ static string container_type_name(const Type* ft) {
 TableValPtr RecordType::GetRecordFieldsVal(const RecordVal* rv) const {
     static auto record_field = id::find_type<RecordType>("record_field");
     static auto record_field_table = id::find_type<TableType>("record_field_table");
-    auto rval = make_intrusive<TableVal>(record_field_table);
+
+    auto attrs = zeek::make_intrusive<detail::Attributes>(record_field_table,
+                                                          /*in_record=*/false,
+                                                          /*is_global=*/false);
+    attrs->AddAttr(make_intrusive<detail::Attr>(detail::ATTR_ORDERED));
+    auto rval = make_intrusive<TableVal>(record_field_table, std::move(attrs));
 
     for ( int i = 0; i < NumFields(); ++i ) {
         const auto& ft = GetFieldType(i);
@@ -1329,17 +1384,14 @@ void RecordType::DescribeFields(ODesc* d, bool func_args) const {
 }
 
 void RecordType::DescribeFieldsReST(ODesc* d, bool func_args) const {
-    if ( ! func_args )
-        d->PushIndent();
-
     for ( int i = 0; i < num_fields; ++i ) {
-        if ( i > 0 ) {
-            if ( func_args )
+        if ( func_args ) {
+            if ( i > 0 )
                 d->Add(", ");
-            else {
-                d->NL();
-                d->NL();
-            }
+        }
+        else {
+            d->NL();
+            d->NL();
         }
 
         const TypeDecl* td = FieldDecl(i);
@@ -1350,8 +1402,39 @@ void RecordType::DescribeFieldsReST(ODesc* d, bool func_args) const {
             if ( num_fields == 1 && util::streq(td->id, "va_args") && td->type->Tag() == TYPE_ANY )
                 // This was a BIF using variable argument list
                 d->Add("...");
-            else
-                td->DescribeReST(d);
+            else {
+                if ( func_args ) {
+                    td->DescribeReST(d);
+                }
+                else {
+                    // ReST rendering of a TypeDecl as zeek:field directive if
+                    // this is about rendering a proper record type and not
+                    // function parameters.
+                    //
+                    // Not sure why we're treating record types and function
+                    // signatures the same thing and than denote what it is
+                    // by passing around a bool. Open-code the field directive
+                    // rendering here.
+                    d->Add(".. zeek:field:: ");
+                    d->Add(td->id);
+                    d->Add(" ");
+                    if ( ! td->type->GetName().empty() ) {
+                        d->Add(":zeek:type:`");
+                        d->Add(td->type->GetName());
+                        d->Add("`");
+                    }
+                    else {
+                        td->type->DescribeReST(d, /*roles_only=*/true);
+                    }
+                    if ( td->attrs ) {
+                        d->SP();
+                        td->attrs->DescribeReST(d, /*shorten=*/true);
+                    }
+
+                    // Good thing ReST doesn't care too much about extra whitespace.
+                    d->NL();
+                }
+            }
         }
 
         if ( func_args )
@@ -1399,11 +1482,8 @@ void RecordType::DescribeFieldsReST(ODesc* d, bool func_args) const {
                 d->Add(cmnts[i].c_str());
         }
 
-        d->PopIndentNoNL();
+        d->PopIndent();
     }
-
-    if ( ! func_args )
-        d->PopIndentNoNL();
 }
 
 string RecordType::GetFieldDeprecationWarning(int field, bool has_check) const {
@@ -1449,7 +1529,7 @@ bool RecordType::IsDeferrable() const {
     // If all creation_inits are deferrable, this record type is deferrable, too.
     // It will be optimized later on. Note, all_of() returns true for an empty
     // range, which is correct.
-    return std::all_of(creation_inits.begin(), creation_inits.end(), is_deferrable);
+    return std::ranges::all_of(creation_inits, is_deferrable);
 }
 
 FileType::FileType(TypePtr yield_type) : Type(TYPE_FILE), yield(std::move(yield_type)) {}
@@ -1638,8 +1718,8 @@ const char* EnumType::Lookup(zeek_int_t value) const {
 
 EnumType::enum_name_list EnumType::Names() const {
     enum_name_list n;
-    for ( auto iter = names.begin(); iter != names.end(); ++iter )
-        n.emplace_back(iter->first, iter->second);
+    for ( const auto& [name, value] : names )
+        n.emplace_back(name, value);
 
     return n;
 }
@@ -1845,7 +1925,7 @@ bool same_type(const Type& arg_t1, const Type& arg_t2, bool is_init, bool match_
     // First do all checks that don't require any recursion.
 
     switch ( t1->Tag() ) {
-        case TYPE_VOID:
+        case TYPE_VOID: // NOLINT(bugprone-branch-clone)
         case TYPE_BOOL:
         case TYPE_INT:
         case TYPE_COUNT:
@@ -2145,8 +2225,7 @@ bool is_assignable(TypeTag t) {
         case TYPE_FUNC:
         case TYPE_ANY:
         case TYPE_ERROR:
-        case TYPE_LIST: return true;
-
+        case TYPE_LIST:
         case TYPE_VECTOR:
         case TYPE_FILE:
         case TYPE_OPAQUE:
@@ -2159,10 +2238,6 @@ bool is_assignable(TypeTag t) {
     return false;
 }
 
-#define CHECK_TYPE(t)                                                                                                  \
-    if ( t1 == t || t2 == t )                                                                                          \
-        return t;
-
 TypeTag max_type(TypeTag t1, TypeTag t2) {
     if ( t1 == TYPE_INTERVAL || t1 == TYPE_TIME )
         t1 = TYPE_DOUBLE;
@@ -2170,9 +2245,12 @@ TypeTag max_type(TypeTag t1, TypeTag t2) {
         t2 = TYPE_DOUBLE;
 
     if ( BothArithmetic(t1, t2) ) {
-        CHECK_TYPE(TYPE_DOUBLE);
-        CHECK_TYPE(TYPE_INT);
-        CHECK_TYPE(TYPE_COUNT);
+        if ( t1 == TYPE_DOUBLE || t2 == TYPE_DOUBLE )
+            return TYPE_DOUBLE;
+        else if ( t1 == TYPE_INT || t2 == TYPE_INT )
+            return TYPE_INT;
+        else if ( t1 == TYPE_COUNT || t2 == TYPE_COUNT )
+            return TYPE_COUNT;
 
         return TYPE_COUNT;
     }
@@ -2328,7 +2406,7 @@ TypePtr merge_record_types(const Type* t1, const Type* t2) {
                 attrs3->AddAttrs(td2->attrs);
 
             attrs3->AddAttr(make_intrusive<detail::Attr>(detail::ATTR_OPTIONAL));
-            auto td_merge = new TypeDecl(util::copy_string(td2->id), std::move(td2->type), attrs3);
+            auto td_merge = new TypeDecl(util::copy_string(td2->id), td2->type, attrs3);
             tdl3->push_back(td_merge);
         }
     }
@@ -2607,7 +2685,7 @@ const TypePtr& base_type(TypeTag tag) {
     if ( ! base_types[tag] ) {
         base_types[tag] = make_intrusive<Type>(tag, true);
         // Give the base types a pseudo-location for easier identification.
-        detail::Location l(type_name(tag), 0, 0, 0, 0);
+        detail::Location l(type_name(tag), 0, 0);
         base_types[tag]->SetLocationInfo(&l);
     }
 
