@@ -15,7 +15,6 @@
 #include "zeek/file_analysis/Manager.h"
 #include "zeek/file_analysis/analyzer/x509/X509.h"
 #include "zeek/file_analysis/analyzer/x509/ocsp_events.bif.h"
-#include "zeek/file_analysis/analyzer/x509/types.bif.h"
 
 // helper function of sk_X509_value to avoid namespace problem
 // sk_X509_value(X,Y) = > SKM_sk_value(X509,X,Y)
@@ -24,7 +23,7 @@ X509* helper_sk_X509_value(const STACK_OF(X509) * certs, int i) { return sk_X509
 
 namespace zeek::file_analysis::detail {
 
-#define OCSP_STRING_BUF_SIZE 2048
+static constexpr size_t OCSP_STRING_BUF_SIZE = 2048;
 
 static bool OCSP_RESPID_bio(OCSP_BASICRESP* basic_resp, BIO* bio) {
 #if ( OPENSSL_VERSION_NUMBER < 0x10100000L ) || defined(LIBRESSL_VERSION_NUMBER)
@@ -126,7 +125,7 @@ bool OCSP::EndOfFile() {
     const unsigned char* ocsp_char = reinterpret_cast<const unsigned char*>(ocsp_data.data());
 
     if ( request ) {
-        OCSP_REQUEST* req = d2i_OCSP_REQUEST(NULL, &ocsp_char, ocsp_data.size());
+        OCSP_REQUEST* req = d2i_OCSP_REQUEST(nullptr, &ocsp_char, ocsp_data.size());
 
         if ( ! req ) {
             reporter->Weird(GetFile(), "openssl_ocsp_request_parse_error");
@@ -137,7 +136,7 @@ bool OCSP::EndOfFile() {
         OCSP_REQUEST_free(req);
     }
     else {
-        OCSP_RESPONSE* resp = d2i_OCSP_RESPONSE(NULL, &ocsp_char, ocsp_data.size());
+        OCSP_RESPONSE* resp = d2i_OCSP_RESPONSE(nullptr, &ocsp_char, ocsp_data.size());
 
         if ( ! resp ) {
             reporter->Weird(GetFile(), "openssl_ocsp_response_parse_error");
@@ -393,7 +392,8 @@ void OCSP::ParseResponse(OCSP_RESPONSE* resp) {
     const ASN1_GENERALIZEDTIME* produced_at = nullptr;
     const STACK_OF(X509)* certs = nullptr;
 
-    int resp_count, num_ext = 0;
+    int resp_count;
+    int num_ext = 0;
     VectorVal* certs_vector = nullptr;
     int len = 0;
 
