@@ -17,6 +17,7 @@
 
 extern char** environ;
 
+#include "zeek/input/Component.h"
 #include "zeek/input/readers/raw/Plugin.h"
 #include "zeek/input/readers/raw/raw.bif.h"
 #include "zeek/threading/SerialTypes.h"
@@ -367,7 +368,7 @@ bool Raw::DoInit(const ReaderInfo& info, int num_fields, const Field* const* fie
     it = info.config.find("offset"); // we want to seek to a given offset inside the file
     if ( it != info.config.end() && ! execute && (Info().mode == MODE_STREAM || Info().mode == MODE_MANUAL) ) {
         std::string offset_s = it->second;
-        offset = strtoll(offset_s.c_str(), 0, 10);
+        offset = strtoll(offset_s.c_str(), nullptr, 10);
     }
     else if ( it != info.config.end() ) {
         Error(
@@ -606,7 +607,6 @@ bool Raw::DoUpdate() {
         }
     }
 
-    std::string line;
     assert((NumFields() == 1 && ! use_stderr) || (NumFields() == 2 && use_stderr));
     for ( ;; ) {
         if ( stdin_towrite > 0 )
