@@ -4,8 +4,6 @@
 
 #include "zeek/RunState.h"
 #include "zeek/analyzer/protocol/pia/PIA.h"
-#include "zeek/analyzer/protocol/tcp/events.bif.h"
-#include "zeek/analyzer/protocol/tcp/types.bif.h"
 #include "zeek/packet_analysis/protocol/tcp/TCPSessionAdapter.h"
 
 using namespace zeek;
@@ -28,21 +26,13 @@ SessionAdapter* TCPAnalyzer::MakeSessionAdapter(Connection* conn) {
 
 zeek::analyzer::pia::PIA* TCPAnalyzer::MakePIA(Connection* conn) { return new analyzer::pia::PIA_TCP(conn); }
 
-bool TCPAnalyzer::BuildConnTuple(size_t len, const uint8_t* data, Packet* packet, ConnTuple& tuple) {
+bool TCPAnalyzer::InitConnKey(size_t len, const uint8_t* data, Packet* packet, IPBasedConnKey& key) {
     uint32_t min_hdr_len = sizeof(struct tcphdr);
     if ( ! CheckHeaderTrunc(min_hdr_len, len, packet) )
         return false;
 
-    tuple.src_addr = packet->ip_hdr->SrcAddr();
-    tuple.dst_addr = packet->ip_hdr->DstAddr();
-
-    data = packet->ip_hdr->Payload();
-
-    const struct tcphdr* tp = (const struct tcphdr*)data;
-    tuple.src_port = tp->th_sport;
-    tuple.dst_port = tp->th_dport;
-    tuple.is_one_way = false;
-    tuple.proto = packet->proto;
+    const struct tcphdr* tp = (const struct tcphdr*)packet->ip_hdr->Payload();
+    key.InitTuple(packet->ip_hdr->SrcAddr(), tp->th_sport, packet->ip_hdr->DstAddr(), tp->th_dport, packet->proto);
 
     return true;
 }
