@@ -43,8 +43,8 @@ TCPSessionAdapter::TCPSessionAdapter(Connection* conn) : packet_analysis::IP::Se
 }
 
 TCPSessionAdapter::~TCPSessionAdapter() {
-    LOOP_OVER_GIVEN_CHILDREN(i, packet_children)
-    delete *i;
+    for ( Analyzer* a : packet_children )
+        delete a;
 
     delete orig;
     delete resp;
@@ -52,18 +52,18 @@ TCPSessionAdapter::~TCPSessionAdapter() {
 
 void TCPSessionAdapter::Init() {
     Analyzer::Init();
-    LOOP_OVER_GIVEN_CHILDREN(i, packet_children)
-    (*i)->Init();
+    for ( Analyzer* a : packet_children )
+        a->Init();
 }
 
 void TCPSessionAdapter::Done() {
-    Analyzer::Done();
+    IP::SessionAdapter::Done();
 
     if ( run_state::terminating && connection_pending && is_active && ! BothClosed() )
         Event(connection_pending);
 
-    LOOP_OVER_GIVEN_CHILDREN(i, packet_children)
-    (*i)->Done();
+    for ( Analyzer* a : packet_children )
+        a->Done();
 
     orig->Done();
     resp->Done();
@@ -192,16 +192,17 @@ static void update_history(analyzer::tcp::TCP_Flags flags, analyzer::tcp::TCP_En
     int bits_set = (flags.SYN() ? 1 : 0) + (flags.FIN() ? 1 : 0) + (flags.RST() ? 1 : 0);
     if ( bits_set > 1 ) {
         if ( flags.FIN() && flags.RST() )
-            endpoint->CheckHistory(HIST_FIN_RST_PKT, 'I');
+            endpoint->CheckHistory(analyzer::tcp::TCP_Endpoint::HIST_FIN_RST_PKT, 'I');
         else
-            endpoint->CheckHistory(HIST_MULTI_FLAG_PKT, 'Q');
+            endpoint->CheckHistory(analyzer::tcp::TCP_Endpoint::HIST_MULTI_FLAG_PKT, 'Q');
     }
 
     else if ( bits_set == 1 ) {
         if ( flags.SYN() ) {
             char code = flags.ACK() ? 'H' : 'S';
 
-            if ( endpoint->CheckHistory(HIST_SYN_PKT, code) && rel_seq != endpoint->hist_last_SYN )
+            if ( endpoint->CheckHistory(analyzer::tcp::TCP_Endpoint::HIST_SYN_PKT, code) &&
+                 rel_seq != endpoint->hist_last_SYN )
                 endpoint->AddHistory(code);
 
             endpoint->hist_last_SYN = rel_seq;
@@ -211,14 +212,16 @@ static void update_history(analyzer::tcp::TCP_Flags flags, analyzer::tcp::TCP_En
             // For FIN's, the sequence number comes at the
             // end of (any data in) the packet, not the
             // beginning as for SYNs and RSTs.
-            if ( endpoint->CheckHistory(HIST_FIN_PKT, 'F') && rel_seq + len != endpoint->hist_last_FIN )
+            if ( endpoint->CheckHistory(analyzer::tcp::TCP_Endpoint::HIST_FIN_PKT, 'F') &&
+                 rel_seq + len != endpoint->hist_last_FIN )
                 endpoint->AddHistory('F');
 
             endpoint->hist_last_FIN = rel_seq + len;
         }
 
         if ( flags.RST() ) {
-            if ( endpoint->CheckHistory(HIST_RST_PKT, 'R') && rel_seq != endpoint->hist_last_RST )
+            if ( endpoint->CheckHistory(analyzer::tcp::TCP_Endpoint::HIST_RST_PKT, 'R') &&
+                 rel_seq != endpoint->hist_last_RST )
                 endpoint->AddHistory('R');
 
             endpoint->hist_last_RST = rel_seq;
@@ -227,10 +230,10 @@ static void update_history(analyzer::tcp::TCP_Flags flags, analyzer::tcp::TCP_En
 
     else { // bits_set == 0
         if ( len )
-            endpoint->CheckHistory(HIST_DATA_PKT, 'D');
+            endpoint->CheckHistory(analyzer::tcp::TCP_Endpoint::HIST_DATA_PKT, 'D');
 
         else if ( flags.ACK() )
-            endpoint->CheckHistory(HIST_ACK_PKT, 'A');
+            endpoint->CheckHistory(analyzer::tcp::TCP_Endpoint::HIST_ACK_PKT, 'A');
     }
 }
 
@@ -299,7 +302,7 @@ static zeek::RecordValPtr build_syn_packet_val(bool is_orig, const zeek::IP_Hdr*
 
     // Parse TCP options.
     u_char* options = (u_char*)tcp + sizeof(struct tcphdr);
-    u_char* opt_end = (u_char*)tcp + tcp->th_off * 4;
+    u_char* opt_end = (u_char*)tcp + static_cast<ptrdiff_t>(tcp->th_off * 4);
 
     while ( options < opt_end ) {
         unsigned int opt = options[0];
@@ -460,23 +463,27 @@ static int32_t update_last_seq(analyzer::tcp::TCP_Endpoint* endpoint, uint32_t l
         // ### perhaps trust RST seq #'s if initial and not too
         // outlandish, but not if they're coming after the other
         // side has sent a FIN - trust the FIN ack instead
-        ;
+        ; // NOLINT(bugprone-branch-clone)
+
 
     else if ( flags.FIN() && endpoint->LastSeq() == endpoint->StartSeq() + 1 )
         // Update last_seq based on the FIN even if delta_last < 0.
         // This is to accommodate > 2 GB connections for which
         // we've only seen the SYN and the FIN (hence the check
         // for last_seq == start_seq + 1).
-        endpoint->UpdateLastSeq(last_seq);
+        endpoint->UpdateLastSeq(last_seq); // NOLINT(bugprone-branch-clone)
+
 
     else if ( endpoint->state == analyzer::tcp::TCP_ENDPOINT_RESET )
         // don't trust any subsequent sequence numbers
-        ;
+        ; // NOLINT(bugprone-branch-clone)
+
 
     else if ( delta_last > 0 )
         // ### check for large jumps here.
         // ## endpoint->last_seq = last_seq;
-        endpoint->UpdateLastSeq(last_seq);
+        endpoint->UpdateLastSeq(last_seq); // NOLINT(bugprone-branch-clone)
+
 
     else if ( delta_last <= 0 && len > 0 )
         endpoint->DidRxmit();
@@ -640,9 +647,8 @@ analyzer::Analyzer* TCPSessionAdapter::FindChild(analyzer::ID arg_id) {
     if ( child )
         return child;
 
-    LOOP_OVER_GIVEN_CHILDREN(i, packet_children) {
-        analyzer::Analyzer* child = (*i)->FindChild(arg_id);
-        if ( child )
+    for ( Analyzer* a : packet_children ) {
+        if ( analyzer::Analyzer* child = a->FindChild(arg_id) )
             return child;
     }
 
@@ -655,9 +661,8 @@ analyzer::Analyzer* TCPSessionAdapter::FindChild(zeek::Tag arg_tag) {
     if ( child )
         return child;
 
-    LOOP_OVER_GIVEN_CHILDREN(i, packet_children) {
-        analyzer::Analyzer* child = (*i)->FindChild(arg_tag);
-        if ( child )
+    for ( Analyzer* a : packet_children ) {
+        if ( analyzer::Analyzer* child = a->FindChild(arg_tag) )
             return child;
     }
 
@@ -1042,8 +1047,8 @@ void TCPSessionAdapter::UpdateConnVal(RecordVal* conn_val) {
     Analyzer::UpdateConnVal(conn_val);
 
     // Have to do packet_children ourselves.
-    LOOP_OVER_GIVEN_CHILDREN(i, packet_children)
-    (*i)->UpdateConnVal(conn_val);
+    for ( Analyzer* a : packet_children )
+        a->UpdateConnVal(conn_val);
 }
 
 void TCPSessionAdapter::AttemptTimer(double /* t */) {
@@ -1140,28 +1145,28 @@ void TCPSessionAdapter::DeleteTimer(double /* t */) { session_mgr->Remove(Conn()
 void TCPSessionAdapter::ConnDeleteTimer(double t) { Conn()->DeleteTimer(t); }
 
 void TCPSessionAdapter::SetContentsFile(unsigned int direction, FilePtr f) {
-    if ( direction == CONTENTS_NONE ) {
+    if ( direction == analyzer::CONTENTS_NONE ) {
         orig->SetContentsFile(nullptr);
         resp->SetContentsFile(nullptr);
     }
 
     else {
-        if ( direction == CONTENTS_ORIG || direction == CONTENTS_BOTH )
+        if ( direction == analyzer::CONTENTS_ORIG || direction == analyzer::CONTENTS_BOTH )
             orig->SetContentsFile(f);
-        if ( direction == CONTENTS_RESP || direction == CONTENTS_BOTH )
+        if ( direction == analyzer::CONTENTS_RESP || direction == analyzer::CONTENTS_BOTH )
             resp->SetContentsFile(f);
     }
 }
 
 FilePtr TCPSessionAdapter::GetContentsFile(unsigned int direction) const {
     switch ( direction ) {
-        case CONTENTS_NONE: return nullptr;
+        case analyzer::CONTENTS_NONE: return nullptr;
 
-        case CONTENTS_ORIG: return orig->GetContentsFile();
+        case analyzer::CONTENTS_ORIG: return orig->GetContentsFile();
 
-        case CONTENTS_RESP: return resp->GetContentsFile();
+        case analyzer::CONTENTS_RESP: return resp->GetContentsFile();
 
-        case CONTENTS_BOTH:
+        case analyzer::CONTENTS_BOTH:
             if ( orig->GetContentsFile() != resp->GetContentsFile() )
                 // This is an "error".
                 return nullptr;
@@ -1178,11 +1183,12 @@ FilePtr TCPSessionAdapter::GetContentsFile(unsigned int direction) const {
 void TCPSessionAdapter::ConnectionClosed(analyzer::tcp::TCP_Endpoint* endpoint, analyzer::tcp::TCP_Endpoint* peer,
                                          bool gen_event) {
     const analyzer::analyzer_list& children(GetChildren());
-    LOOP_OVER_CONST_CHILDREN(i)
-    // Using this type of cast here is nasty (will crash if
-    // we inadvertently have a child analyzer that's not a
-    // TCP_ApplicationAnalyzer), but we have to ...
-    static_cast<analyzer::tcp::TCP_ApplicationAnalyzer*>(*i)->ConnectionClosed(endpoint, peer, gen_event);
+
+    for ( Analyzer* a : children )
+        // Using this type of cast here is nasty (will crash if
+        // we inadvertently have a child analyzer that's not a
+        // TCP_ApplicationAnalyzer), but we have to ...
+        static_cast<analyzer::tcp::TCP_ApplicationAnalyzer*>(a)->ConnectionClosed(endpoint, peer, gen_event);
 
     if ( DataPending(endpoint) ) {
         // Don't close out the connection yet, there's still data to
@@ -1262,9 +1268,10 @@ void TCPSessionAdapter::ConnectionClosed(analyzer::tcp::TCP_Endpoint* endpoint,
 
 void TCPSessionAdapter::ConnectionFinished(bool half_finished) {
     const analyzer::analyzer_list& children(GetChildren());
-    LOOP_OVER_CONST_CHILDREN(i)
-    // Again, nasty - see TCPSessionAdapter::ConnectionClosed.
-    static_cast<analyzer::tcp::TCP_ApplicationAnalyzer*>(*i)->ConnectionFinished(half_finished);
+
+    for ( Analyzer* a : children )
+        // Again, nasty - see TCPSessionAdapter::ConnectionClosed.
+        static_cast<analyzer::tcp::TCP_ApplicationAnalyzer*>(a)->ConnectionFinished(half_finished);
 
     if ( half_finished )
         Event(connection_half_finished);
@@ -1278,8 +1285,8 @@ void TCPSessionAdapter::ConnectionReset() {
     Event(connection_reset);
 
     const analyzer::analyzer_list& children(GetChildren());
-    LOOP_OVER_CONST_CHILDREN(i)
-    static_cast<analyzer::tcp::TCP_ApplicationAnalyzer*>(*i)->ConnectionReset();
+    for ( Analyzer* a : children )
+        static_cast<analyzer::tcp::TCP_ApplicationAnalyzer*>(a)->ConnectionReset();
 
     is_active = 0;
 }
@@ -1308,8 +1315,8 @@ void TCPSessionAdapter::EndpointEOF(analyzer::tcp::TCP_Reassembler* endp) {
         EnqueueConnEvent(connection_EOF, ConnVal(), val_mgr->Bool(endp->IsOrig()));
 
     const analyzer::analyzer_list& children(GetChildren());
-    LOOP_OVER_CONST_CHILDREN(i)
-    static_cast<analyzer::tcp::TCP_ApplicationAnalyzer*>(*i)->EndpointEOF(endp->IsOrig());
+    for ( Analyzer* a : children )
+        static_cast<analyzer::tcp::TCP_ApplicationAnalyzer*>(a)->EndpointEOF(endp->IsOrig());
 
     if ( close_deferred ) {
         if ( DataPending(endp->Endpoint()) ) {
@@ -1327,8 +1334,8 @@ void TCPSessionAdapter::EndpointEOF(analyzer::tcp::TCP_Reassembler* endp) {
 
 void TCPSessionAdapter::PacketWithRST() {
     const analyzer::analyzer_list& children(GetChildren());
-    LOOP_OVER_CONST_CHILDREN(i)
-    static_cast<analyzer::tcp::TCP_ApplicationAnalyzer*>(*i)->PacketWithRST();
+    for ( Analyzer* a : children )
+        static_cast<analyzer::tcp::TCP_ApplicationAnalyzer*>(a)->PacketWithRST();
 }
 
 void TCPSessionAdapter::CheckPIA_FirstPacket(bool is_orig, const IP_Hdr* ip) {
@@ -1458,7 +1465,7 @@ void TCPSessionAdapter::SynWeirds(analyzer::tcp::TCP_Flags flags, analyzer::tcp:
 int TCPSessionAdapter::ParseTCPOptions(const struct tcphdr* tcp, bool is_orig) {
     // Parse TCP options.
     const u_char* options = (const u_char*)tcp + sizeof(struct tcphdr);
-    const u_char* opt_end = (const u_char*)tcp + tcp->th_off * 4;
+    const u_char* opt_end = (const u_char*)tcp + static_cast<ptrdiff_t>(tcp->th_off * 4);
     std::vector<const u_char*> opts;
 
     while ( options < opt_end ) {
