@@ -6,7 +6,6 @@
 #include <openssl/evp.h>
 #include <typeinfo>
 
-#include "zeek/NetVar.h"
 #include "zeek/Var.h"
 #include "zeek/broker/Data.h"
 #include "zeek/digest.h"
@@ -69,9 +68,9 @@ std::unique_ptr<Hasher> Hasher::Unserialize(BrokerDataView data) {
     std::unique_ptr<Hasher> hasher;
 
     switch ( type ) {
-        case Default: hasher.reset(new DefaultHasher(k, {h1, h2})); break;
-
-        case Double: hasher.reset(new DoubleHasher(k, {h1, h2})); break;
+        case Default: hasher = std::make_unique<DefaultHasher>(k, Hasher::seed_t{h1, h2}); break;
+        case Double: hasher = std::make_unique<DoubleHasher>(k, Hasher::seed_t{h1, h2}); break;
+        default: break;
     }
 
     // Note that the derived classed don't hold any further state of
