@@ -2,7 +2,6 @@
 
 #include "zeek/script_opt/Inline.h"
 
-#include "zeek/Desc.h"
 #include "zeek/EventRegistry.h"
 #include "zeek/module_util.h"
 #include "zeek/script_opt/Expr.h"
@@ -282,7 +281,7 @@ void Inliner::CoalesceEventHandlers(ScriptFuncPtr func, const std::vector<Func::
             return;
 
         auto ie_s = with_location_of(make_intrusive<ExprStmt>(ie), bp);
-        merged_body->Stmts().push_back(std::move(ie_s));
+        merged_body->Stmts().emplace_back(std::move(ie_s));
     }
 
     auto inlined_func = make_intrusive<CoalescedScriptFunc>(merged_body, new_scope, func);
