@@ -25,7 +25,6 @@
 #include <cstdarg>
 #include <cstdio>
 #include <cstdlib>
-#include <sstream>
 
 #if defined(HAVE_MALLINFO) || defined(HAVE_MALLINFO2)
 #include <malloc.h>
@@ -40,25 +39,29 @@
 
 #include <algorithm>
 #include <array>
+#include <filesystem>
 #include <iostream>
-#include <random>
+#include <ranges>
+#include <sstream>
 #include <string>
 #include <vector>
 
 #include "zeek/3rdparty/ConvertUTF.h"
 #include "zeek/Desc.h"
 #include "zeek/Hash.h"
-#include "zeek/NetVar.h"
 #include "zeek/Obj.h"
 #include "zeek/Reporter.h"
 #include "zeek/RunState.h"
-#include "zeek/ScannedFile.h"
 #include "zeek/Val.h"
 #include "zeek/digest.h"
 #include "zeek/input.h"
 #include "zeek/iosource/Manager.h"
 #include "zeek/iosource/PktSrc.h"
 
+#ifdef _MSC_VER
+#include "zeek/ScannedFile.h"
+#endif
+
 #include "zeek/3rdparty/doctest.h"
 
 using namespace std;
@@ -89,11 +92,10 @@ std::string extract_ip(const std::string& i) {
     if ( s.size() > 0 && s[0] == '[' )
         s.erase(0, 1);
 
-    if ( s.size() > 1 && s.substr(0, 2) == "0x" )
+    if ( s.starts_with("0x") )
         s.erase(0, 2);
 
-    size_t pos = 0;
-    if ( (pos = s.find(']')) != std::string::npos )
+    if ( size_t pos = s.find(']'); pos != std::string::npos )
         s = s.substr(0, pos);
 
     return s;
@@ -297,9 +299,9 @@ void hmac_md5(size_t size, const unsigned char* bytes, unsigned char digest[16])
 
 static bool read_random_seeds(const char* read_file, uint32_t* seed,
                               std::array<uint32_t, zeek::detail::KeyedHash::SEED_INIT_SIZE>& buf) {
-    FILE* f = nullptr;
+    FILE* f = fopen(read_file, "r");
 
-    if ( ! (f = fopen(read_file, "r")) ) {
+    if ( ! f ) {
         reporter->Warning("Could not open seed file '%s': %s", read_file, strerror(errno));
         return false;
     }
@@ -327,9 +329,9 @@ static bool read_random_seeds(const char* read_file, uint32_t* seed,
 
 static bool write_random_seeds(const char* write_file, uint32_t seed,
                                std::array<uint32_t, zeek::detail::KeyedHash::SEED_INIT_SIZE>& buf) {
-    FILE* f = nullptr;
+    FILE* f = fopen(write_file, "w+");
 
-    if ( ! (f = fopen(write_file, "w+")) ) {
+    if ( ! f ) {
         reporter->Warning("Could not create seed file '%s': %s", write_file, strerror(errno));
         return false;
     }
@@ -409,7 +411,7 @@ void init_random_seed(const char* read_file, const char* write_file, bool use_em
         pos += nbytes / sizeof(uint32_t);
 #else
         // Gather up some entropy.
-        gettimeofday((struct timeval*)(buf.data() + pos), 0);
+        gettimeofday((struct timeval*)(buf.data() + pos), nullptr);
         pos += sizeof(struct timeval) / sizeof(uint32_t);
 
         // use urandom. For reasons see e.g. http://www.2uo.de/myths-about-urandom/
@@ -531,19 +533,6 @@ FILE* open_package(string& path, const string& mode) {
     return nullptr;
 }
 
-void SafePathOp::CheckValid(const char* op_result, const char* path, bool error_aborts) {
-    if ( op_result ) {
-        result = op_result;
-        error = false;
-    }
-    else {
-        if ( error_aborts )
-            reporter->InternalError("Path operation failed on %s: %s", path ? path : "<null>", strerror(errno));
-        else
-            error = true;
-    }
-}
-
 TEST_CASE("util flatten_script_name") {
     CHECK(flatten_script_name("script", "some/path") == "some.path.script");
 #ifndef _MSC_VER
@@ -611,10 +600,10 @@ string normalize_path(std::string_view path) {
     }
     // "//" interferes with std::weakly_canonical
     string stringPath = string(path);
-    if ( stringPath._Starts_with("//") ) {
+    if ( stringPath.starts_with("//") ) {
         stringPath.erase(0, 2);
     }
-    return zeek::filesystem::path(stringPath).lexically_normal().string();
+    return std::filesystem::path(stringPath).lexically_normal().string();
 #else
     if ( path.find("/.") == std::string_view::npos && path.find("//") == std::string_view::npos ) {
         // no need to normalize anything
@@ -664,8 +653,8 @@ string normalize_path(std::string_view path) {
         }
     }
 
-    for ( auto it = final_components.begin(); it != final_components.end(); ++it ) {
-        new_path.append(*it);
+    for ( const auto& comp : final_components ) {
+        new_path.append(comp);
         new_path.append("/");
     }
 
@@ -681,10 +670,10 @@ string without_zeekpath_component(std::string_view path) {
 
     const auto paths = tokenize_string(zeek_path(), path_list_separator[0]);
 
-    for ( size_t i = 0; i < paths.size(); ++i ) {
-        string common = normalize_path(paths[i]);
+    for ( const auto& p : paths ) {
+        string common = normalize_path(p);
 
-        if ( rval.find(common) != 0 )
+        if ( ! rval.starts_with(common) )
             continue;
 
         // Found the containing directory.
@@ -704,7 +693,7 @@ string without_zeekpath_component(std::string_view path) {
 std::string get_exe_path(const std::string& invocation) {
     if ( invocation.empty() )
         return "";
-    zeek::filesystem::path invocation_path(invocation);
+    std::filesystem::path invocation_path(invocation);
 
     if ( invocation_path.is_absolute() || invocation_path.root_directory() == "~" )
         // Absolute path
@@ -719,7 +708,7 @@ std::string get_exe_path(const std::string& invocation) {
             exit(1);
         }
 
-        return (zeek::filesystem::path(cwd) / invocation_path).string();
+        return (std::filesystem::path(cwd) / invocation_path).string();
     }
 
     auto path = getenv("PATH");
@@ -844,7 +833,7 @@ void set_processing_status(const char* status, const char* reason) {
     auto write_str = [](int fd, const char* s) {
         int len = strlen(s);
         while ( len ) {
-            int n = write(fd, s, len);
+            ssize_t n = write(fd, s, len);
 
             if ( n < 0 && errno != EINTR && errno != EAGAIN )
                 // Ignore errors, as they're too difficult to
@@ -1046,31 +1035,8 @@ TEST_CASE("util streq") {
 
 bool streq(const char* s1, const char* s2) { return strcmp(s1, s2) == 0; }
 
-bool starts_with(std::string_view s, std::string_view beginning) {
-    if ( beginning.size() > s.size() )
-        return false;
-
-    return std::equal(beginning.begin(), beginning.end(), s.begin());
-}
-
-TEST_CASE("util starts_with") {
-    CHECK(starts_with("abcde", "ab") == true);
-    CHECK(starts_with("abcde", "de") == false);
-    CHECK(starts_with("abcde", "abcedf") == false);
-}
-
-bool ends_with(std::string_view s, std::string_view ending) {
-    if ( ending.size() > s.size() )
-        return false;
-
-    return std::equal(ending.rbegin(), ending.rend(), s.rbegin());
-}
-
-TEST_CASE("util ends_with") {
-    CHECK(ends_with("abcde", "de") == true);
-    CHECK(ends_with("abcde", "fg") == false);
-    CHECK(ends_with("abcde", "abcedf") == false);
-}
+bool starts_with(std::string_view s, std::string_view beginning) { return s.starts_with(beginning); }
+bool ends_with(std::string_view s, std::string_view ending) { return s.ends_with(ending); }
 
 char* skip_whitespace(char* s) {
     while ( *s == ' ' || *s == '\t' )
@@ -1181,7 +1147,7 @@ void to_upper(char* s) {
 
 string to_upper(const std::string& s) {
     string t = s;
-    std::transform(t.begin(), t.end(), t.begin(), ::toupper);
+    std::ranges::transform(t, t.begin(), ::toupper);
     return t;
 }
 
@@ -1348,7 +1314,6 @@ char* uitoa_n(uint64_t value, char* str, int n, int base, const char* prefix) {
 
     int i = 0;
     uint64_t v;
-    char *p, *q;
     char c;
 
     if ( prefix ) {
@@ -1428,7 +1393,7 @@ TEST_CASE("util strtolower") {
 
 std::string strtolower(const std::string& s) {
     std::string t = s;
-    std::transform(t.begin(), t.end(), t.begin(), ::tolower);
+    std::ranges::transform(t, t.begin(), ::tolower);
     return t;
 }
 
@@ -1442,7 +1407,7 @@ TEST_CASE("util strtoupper") {
 
 std::string strtoupper(const std::string& s) {
     std::string t = s;
-    std::transform(t.begin(), t.end(), t.begin(), ::toupper);
+    std::ranges::transform(t, t.begin(), ::toupper);
     return t;
 }
 
@@ -1567,13 +1532,20 @@ TEST_CASE("util strstrip") {
     CHECK(strstrip(s) == "abcd");
 
     s = "  abcd  ";
-    CHECK(strstrip(s) == "abcd");
+    CHECK(strstrip(std::move(s)) == "abcd");
 }
 
 std::string strstrip(std::string s) {
     auto notspace = [](unsigned char c) { return ! std::isspace(c); };
-    s.erase(s.begin(), std::find_if(s.begin(), s.end(), notspace));
+    s.erase(s.begin(), std::ranges::find_if(s, notspace));
+
+    // We require `std::reverse_iterator::base` here which in e.g., gcc-10.2.1
+    // is not implemented for the range equivalent of the code
+    // (`borrowed_iterator_t` over a `reverse_view`). Stick to the non-ranges
+    // version for now.
+    // NOLINTNEXTLINE(modernize-use-ranges)
     s.erase(std::find_if(s.rbegin(), s.rend(), notspace).base(), s.end());
+
     return s;
 }
 
@@ -1606,7 +1578,7 @@ const char* zeek_plugin_path() {
     const char* path = getenv("ZEEK_PLUGIN_PATH");
 
     if ( ! path )
-        path = BRO_PLUGIN_INSTALL_PATH;
+        path = ZEEK_PLUGIN_INSTALL_PATH;
 
     return path;
 }
@@ -1647,55 +1619,6 @@ FILE* open_file(const string& path, const string& mode) {
     return rval;
 }
 
-TEST_CASE("util path ops") {
-#ifdef _MSC_VER
-// TODO: adapt these tests to Windows paths
-#else
-    SUBCASE("SafeDirname") {
-        SafeDirname d("/this/is/a/path", false);
-        CHECK(d.result == "/this/is/a");
-
-        SafeDirname d2("invalid", false);
-        CHECK(d2.result == ".");
-
-        SafeDirname d3("./filename", false);
-        CHECK(d2.result == ".");
-    }
-
-    SUBCASE("SafeBasename") {
-        SafeBasename b("/this/is/a/path", false);
-        CHECK(b.result == "path");
-        CHECK(! b.error);
-
-        SafeBasename b2("justafile", false);
-        CHECK(b2.result == "justafile");
-        CHECK(! b2.error);
-    }
-#endif
-}
-
-SafeDirname::SafeDirname(const char* path, bool error_aborts) : SafePathOp() { DoFunc(path ? path : "", error_aborts); }
-
-SafeDirname::SafeDirname(const string& path, bool error_aborts) : SafePathOp() { DoFunc(path, error_aborts); }
-
-void SafeDirname::DoFunc(const string& path, bool error_aborts) {
-    char* tmp = copy_string(path.c_str());
-    CheckValid(dirname(tmp), tmp, error_aborts);
-    delete[] tmp;
-}
-
-SafeBasename::SafeBasename(const char* path, bool error_aborts) : SafePathOp() {
-    DoFunc(path ? path : "", error_aborts);
-}
-
-SafeBasename::SafeBasename(const string& path, bool error_aborts) : SafePathOp() { DoFunc(path, error_aborts); }
-
-void SafeBasename::DoFunc(const string& path, bool error_aborts) {
-    char* tmp = copy_string(path.c_str());
-    CheckValid(basename(tmp), tmp, error_aborts);
-    delete[] tmp;
-}
-
 TEST_CASE("util implode_string_vector") {
     std::vector<std::string> v = {"a", "b", "c"};
     CHECK(implode_string_vector(v, ",") == "a,b,c");
@@ -1763,7 +1686,7 @@ vector<string>* tokenize_string(std::string_view input, std::string_view delim,
     return rval;
 }
 
-vector<std::string_view> tokenize_string(std::string_view input, const char delim) noexcept {
+vector<std::string_view> tokenize_string(std::string_view input, const char delim) {
     vector<std::string_view> rval;
 
     size_t pos = 0;
@@ -1782,7 +1705,7 @@ static string find_file_in_path(const string& filename, const string& path, cons
     if ( filename.empty() )
         return {};
 
-    zeek::filesystem::path filepath(filename);
+    std::filesystem::path filepath(filename);
 
     // If file name is an absolute path, searching within *path* is pointless.
     if ( filepath.is_absolute() ) {
@@ -1792,7 +1715,7 @@ static string find_file_in_path(const string& filename, const string& path, cons
             return {};
     }
 
-    auto abs_path = (zeek::filesystem::path(path) / filepath).string();
+    auto abs_path = (std::filesystem::path(path) / filepath).string();
 
     if ( ! opt_ext.empty() ) {
         for ( const string& ext : opt_ext ) {
@@ -1817,8 +1740,8 @@ string find_file(const string& filename, const string& path_set, const string& o
     if ( ! opt_ext.empty() )
         ext.push_back(opt_ext);
 
-    for ( size_t n = 0; n < paths.size(); ++n ) {
-        string f = find_file_in_path(filename, paths[n], ext);
+    for ( const auto& p : paths ) {
+        string f = find_file_in_path(filename, p, ext);
 
         if ( ! f.empty() )
             return f;
@@ -1833,8 +1756,8 @@ string find_script_file(const string& filename, const string& path_set) {
 
     vector<string> ext = {".zeek"};
 
-    for ( size_t n = 0; n < paths.size(); ++n ) {
-        string f = find_file_in_path(filename, paths[n], ext);
+    for ( const auto& p : paths ) {
+        string f = find_file_in_path(filename, p, ext);
 
         if ( ! f.empty() )
             return f;
@@ -1853,7 +1776,7 @@ double current_time(bool real) {
     tv.tv_sec = ms.count() / 1000;
     tv.tv_usec = (ms.count() % 1000) * 1000;
 #else
-    if ( gettimeofday(&tv, 0) < 0 )
+    if ( gettimeofday(&tv, nullptr) < 0 )
         reporter->InternalError("gettimeofday failed in current_time()");
 #endif
     double t = double(tv.tv_sec) + double(tv.tv_usec) / 1e6;
@@ -1875,9 +1798,9 @@ double current_time(bool real) {
 struct timeval double_to_timeval(double t) {
     struct timeval tv;
 
-    double t1 = floor(t);
-    tv.tv_sec = int(t1);
-    tv.tv_usec = int((t - t1) * 1e6 + 0.5);
+    double t1 = std::floor(t);
+    tv.tv_sec = static_cast<time_t>(t1);
+    tv.tv_usec = std::lround((t - t1) * 1e6);
 
     return tv;
 }
@@ -1942,7 +1865,7 @@ uint64_t calculate_unique_id(size_t pool) {
             memset(&unique, 0, sizeof(unique)); // Make valgrind happy.
             gethostname(unique.hostname, 120);
             unique.hostname[sizeof(unique.hostname) - 1] = '\0';
-            gettimeofday(&unique.time, 0);
+            gettimeofday(&unique.time, nullptr);
             unique.pool = (uint64_t)pool;
             unique.pid = getpid();
             unique.rnd = static_cast<int>(detail::random_number());
@@ -1967,7 +1890,7 @@ uint64_t calculate_unique_id(size_t pool) {
 
 bool safe_write(int fd, const char* data, int len) {
     while ( len > 0 ) {
-        int n = write(fd, data, len);
+        ssize_t n = write(fd, data, len);
 
         if ( n < 0 ) {
             if ( errno == EINTR )
@@ -2336,6 +2259,7 @@ TEST_CASE("util json_escape_utf8") {
 
     // Valid ASCII and valid ASCII control characters
     CHECK(json_escape_utf8("a") == "a");
+    // NOLINTNEXTLINE(bugprone-string-literal-with-embedded-nul)
     CHECK(json_escape_utf8("\b\f\n\r\t\x00\x15") == "\b\f\n\r\t\x00\x15");
 
     // Table 3-7 in https://www.unicode.org/versions/Unicode12.0.0/ch03.pdf describes what is
@@ -2402,14 +2326,10 @@ static bool check_ok_utf8(const unsigned char* start, const unsigned char* end)
     if ( result != conversionOK )
         return false;
 
-    if ( (output[0] <= 0x001F) || (output[0] == 0x007F) || (output[0] >= 0x0080 && output[0] <= 0x009F) )
-        // Control characters
-        return false;
-    else if ( output[0] >= 0xE000 && output[0] <= 0xF8FF )
-        // Private Use Area
-        return false;
-    else if ( output[0] >= 0xFFF0 && output[0] <= 0xFFFF )
-        // Specials Characters
+    if ( ((output[0] <= 0x001F) || (output[0] == 0x007F) ||
+          (output[0] >= 0x0080 && output[0] <= 0x009F)) || // Control characters
+         (output[0] >= 0xE000 && output[0] <= 0xF8FF) ||   // Private Use Area
+         (output[0] >= 0xFFF0 && output[0] <= 0xFFFF) )    // Special characters
         return false;
 
     return true;
@@ -2491,19 +2411,19 @@ TEST_CASE("util filesystem") {
 #ifdef _MSC_VER
     // TODO: adapt these tests to Windows paths
 #else
-    zeek::filesystem::path path1("/a/b");
+    std::filesystem::path path1("/a/b");
     CHECK(path1.is_absolute());
     CHECK(! path1.is_relative());
     CHECK(path1.filename() == "b");
     CHECK(path1.parent_path() == "/a");
 
-    zeek::filesystem::path path2("/a//b//conn.log");
+    std::filesystem::path path2("/a//b//conn.log");
     CHECK(path2.lexically_normal() == "/a/b/conn.log");
 
-    zeek::filesystem::path path3("a//b//");
+    std::filesystem::path path3("a//b//");
     CHECK(path3.lexically_normal() == "a/b/");
 
-    auto info = zeek::filesystem::space(".");
+    auto info = std::filesystem::space(".");
     CHECK(info.capacity > 0);
 #endif
 }
