@@ -3,6 +3,7 @@
 #include "zeek/zeekygen/utils.h"
 
 #include <sys/stat.h>
+#include <algorithm>
 #include <cerrno>
 
 #include "zeek/Func.h"
@@ -112,11 +113,8 @@ size_t end_of_first_sentence(const string& s) {
 }
 
 bool is_all_whitespace(const string& s) {
-    for ( size_t i = 0; i < s.size(); ++i )
-        if ( ! isspace(s[i]) )
-            return false;
-
-    return true;
+    auto it = std::ranges::find_if(s, [](char c) { return ! isspace(c); });
+    return it == s.end();
 }
 
 string redef_indication(const string& from_script) {
@@ -175,7 +173,7 @@ std::optional<std::string> source_code_range(const zeek::detail::ID* id) {
             if ( id->IsType() ) {
                 loc = type->GetLocationInfo();
 
-                if ( zeek::util::ends_with(loc->filename, ".bif.zeek") )
+                if ( std::string_view{loc->FileName()}.ends_with(".bif.zeek") )
                     // Source code won't be available to reference, so fall back
                     // to identifier location which may actually be in a regular
                     // .zeek script.
@@ -193,8 +191,8 @@ std::optional<std::string> source_code_range(const zeek::detail::ID* id) {
     if ( loc == &zeek::detail::no_location )
         return {};
 
-    return util::fmt("%s %d %d", normalize_script_path(loc->filename).data(), loc->first_line,
-                     loc->last_line + extra_lines);
+    return util::fmt("%s %d %d", normalize_script_path(loc->FileName()).data(), loc->FirstLine(),
+                     loc->LastLine() + extra_lines);
 }
 
 } // namespace zeek::zeekygen::detail
